package ar.com.unobtainium.tonetennis.scenes 
{
	import ar.com.unobtainium.tonetennis.elements.Ball;
	import ar.com.unobtainium.tonetennis.elements.Pad;
	import ar.com.unobtainium.tonetennis.elements.PixelTextField;
	import ar.com.unobtainium.tonetennis.elements.Player;
	import ar.com.unobtainium.tonetennis.elements.ScoreDisplay;
	import ar.com.unobtainium.tonetennis.sound.SoundMan;
	import com.greensock.data.TweenMaxVars;
	import com.greensock.events.TweenEvent;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.TimerEvent;
	import flash.text.TextFormat;
	import flash.utils.Timer;
	
	import com.greensock.*;
	import com.greensock.easing.*;
	
	/**
	 * Class to have a 1v1 match of Tone Tennis
	 * 
	 * This class handles the rules and gameplay loop. It is the base for gameplay
	 * In the future it will also probably manage the audiovisual aspects?
	 * 
	 * @author Balthazar Auger
	 */
	public class Gameplay extends Sprite 
	{
		
		public var pads:Array = [110, 380, 640, 900, 1170];
		public var padColors:Array = [0xffd580, 0xfffa88, 0xbefa8b, 0xb0ffd2, 0x64e0ff];
		
		
		public const EMPTY_TEXT:String = "";
		public const SERVE_TEXT:String = "SERVE";
		public const HIT_TEXT:String = "HIT";
		public const SMASH_TEXT:String = "SMASH";
		public const START_CATCH_TEXT:String = "CATCH";
		public const END_CATCH_TEXT:String = "HOLD";
		
		public const BG_COLOR:uint = 0x333333;
		
		public var ball:Ball;
		
		public var player0:Player = new Player(0);
		public var player1:Player = new Player(1);
		
		
		public var toPoints:int;
		/**
		 * Keeps track of current gameplay speed level. LOWER IS FASTER
		 */
		public var speedLevel:int;
		public const MAX_SPEED_LEVEL:int = 6; //slowest
		public const MIN_SPEED_LEVEL:int = 0; //fastest
		
		public const VOLLEY_DISTANCE:int = 1500;
		
		public const BOOST_FACTOR:int = 2;
		
		protected var keys:Object = { };
		
		protected var ballOutTimer:Timer;
		
		protected var soundManager:SoundMan;
		
		public function Gameplay(maxPoints:int) 
		{
			super();
			
			toPoints = maxPoints;
			
			addEventListener(Event.ADDED_TO_STAGE, sceneAdded);

		}
		
		/**
		 * Gameplay Entry point
		 * @param	evt
		 */
		protected function sceneAdded(evt:Event):void
		{
			draw();
			
			initKeyboard();
			
			startMatch();
		}
		
		/**
		 * Creates the components of this scene:
			 * 5 pads
			 * 2 players
			 * 2 score displays
			 * 1 ball
		 */
		protected function draw():void
		{
			//draw background
			var bg:Sprite = new Sprite();
			bg.graphics.beginFill(BG_COLOR);
			bg.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			bg.graphics.endFill();
			this.addChild(bg);
			
			//add pads
			for (var i:int = 0; i < pads.length;i++)
			{
				var pad:Pad = new Pad(padColors[i]);
				pad.x = pads[i];
				this.addChild(pad);
				pads[i] = pad;
			}
			
			//add score counters
			var sc0:ScoreDisplay = new ScoreDisplay();
			var sc1:ScoreDisplay = new ScoreDisplay();
			
			sc0.y = sc1.y = 30;
			sc0.x = pads[0].x;
			sc1.x = pads[4].x;
			
			player0.score = sc0; player1.score = sc1;
			
			this.addChild(sc0); this.addChild(sc1);
			
			//create sound manager
			soundManager = new SoundMan();
		}
		
		protected function startMatch():void
		{
			speedLevel = MIN_SPEED_LEVEL;
			
			//for now we'll assume the left player always kicks off first
			//player0.state = Player.STATE_SERVE;
			//player1.state = Player.STATE_DEFENSE;
			player0.opponent = player1;
			player1.opponent = player0;

			//give ball to left player
			giveBall(player1);
			
			prepServe(player1);
			
			//begin ball out of court watch
			ballOutTimer = new Timer(500);
			ballOutTimer.addEventListener(TimerEvent.TIMER, ballOutTick);
			ballOutTimer.start();
		}
		
		protected function giveBall(player:Player):void
		{
			ball = new Ball();
			
			var homePad:Pad;
			if (player.playerNum == 0)
				homePad = pads[0];
			else
				homePad = pads[4];
			
			ball.x = homePad.x;		
			ball.y = 380;
			this.addChild(ball);
		}
		
		protected function ballOutTick(te:TimerEvent):void
		{
			
			if (ball.x < 0)
			{
				//point for right player - do score function
				scorePoint(player1);
			}
			else if (ball.x > stage.stageWidth)
			{
				//point for left player
				scorePoint(player0);
			}
		}
		
		/**
		 * KEYBOARD INPUT
		 */
		protected function initKeyboard():void
		{
			this.stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			this.stage.addEventListener(KeyboardEvent.KEY_UP, handleKeyUp);
		}
		
		protected function handleKeyDown(ke:KeyboardEvent):void
		{
			//trace (ke.keyCode + " down");
			 
			if( ! Boolean(ke.keyCode in keys)) {  
				keys[ke.keyCode] = true;
				

				if ((ke.keyCode >= 49) && (ke.keyCode <= 53))
				{
					actOnPadDown(ke.keyCode - 49);					
				}
			}	
		}
		
		protected function handleKeyUp(ke:KeyboardEvent):void
		{
			//trace (ke.keyCode + " up");
			delete keys[ke.keyCode];
			
			if ((ke.keyCode >= 49) && (ke.keyCode <= 53))
			{
				var releasedPad:Pad = (pads[ke.keyCode - 49] as Pad);
				releasedPad.release();
			}
			
		}
		
		protected function actOnPadDown(padNum:int):void
		{	
			//satisfy action feedback
			var pressedPad:Pad = (pads[padNum] as Pad);
			pressedPad.press();
			
			switch (padNum)
			{
				case 0: //left player home pad
					switch (player0.state)
					{
						case Player.STATE_NONE:
							//if hits ball, start kickoff sequence
							if (pressedPad.collidesWith(ball))
								startServe(player0);
							break;						
						case Player.STATE_SERVE:
							//nothing
							break;
						case Player.STATE_DEFENSE:
							//simple volley
							if (pressedPad.collidesWith(ball))
								volley(player0);
							break;
						case Player.STATE_CATCHING:
							//if hits ball, ball is caugth, go to kickoff
							if (pressedPad.collidesWith(ball))
								catchBall(player0);
							break;
						case Player.STATE_ATTACK:
							//do nothing
							break;
					}
					break;
				case 1: // left player reach pad
					switch (player0.state)
					{
						case Player.STATE_SERVE:
							//Serve ball, go to defense
							if (pressedPad.collidesWith(ball))
								serve(player0);
							break;
						case Player.STATE_DEFENSE:
							//if hits ball, go to catching. Else, do nothing
							if (pressedPad.collidesWith(ball))
								tryCatch(player0);
							break;
						case Player.STATE_CATCHING:
							//do nothing
							break;
						case Player.STATE_ATTACK:
							//do nothing
							break;
					}
					break;
				case 2: // net pad. if hit, make the ball go faster
					if (pressedPad.collidesWith(ball))
						boostBall();
					break;
				case 3: // right player reach pad
					switch (player1.state)
					{
						case Player.STATE_SERVE:
							//Serve ball, go to defense
							if (pressedPad.collidesWith(ball))
								serve(player1);
							break;
						case Player.STATE_DEFENSE:
							//if hits ball, go to catching. Else, do nothing
							if (pressedPad.collidesWith(ball))
								tryCatch(player1);
							break;
						case Player.STATE_CATCHING:
							//do nothing
							break;
						case Player.STATE_ATTACK:
							//do nothing
							break;
					}
					break;
				case 4: // right player home pad
					switch (player1.state)
					{
						case Player.STATE_NONE:
							//if hits ball, start kickoff sequence
							if (pressedPad.collidesWith(ball))
								startServe(player1);
							break;	
						case Player.STATE_SERVE:
							//if hits ball, start kickoff sequence
							break;
						case Player.STATE_DEFENSE:
							//simple volley
							if (pressedPad.collidesWith(ball))
								volley(player1);
							break;
						case Player.STATE_CATCHING:
							//if hits ball, ball is caugth, go to kickoff
							if (pressedPad.collidesWith(ball))
								catchBall(player1);
							break;
						case Player.STATE_ATTACK:
							//do nothing
							break;
					}
					break;
			}
		}
		
		protected function scorePoint(player:Player):void
		{
			this.removeChild(ball);
			
			soundManager.dimHums();
			
			//increment score for player
			
			player.score.addPoint();
			
			if (player.score.points +1 == toPoints)
			{
				playerWins(player);
				return;
			}
			
			//give ball to player who scored (like voleyball)
			giveBall(player);
			
			//prep new serve
			prepServe(player);
		}
		
		protected function prepServe(player:Player):void
		{
			
			player.state = Player.STATE_NONE;
			player.opponent.state = Player.STATE_NONE;
			
			ball.speed = ball.MAX_SPEED;
			speedLevel = MIN_SPEED_LEVEL;
			
			for each (var pad:Pad in pads)
			{
				pad.restoreWidth();
			}
			
			if (player.playerNum == 0)
			{
				(pads[0] as Pad).setText(SERVE_TEXT);
				(pads[1] as Pad).setText(EMPTY_TEXT);
				(pads[2] as Pad).setText(EMPTY_TEXT);
				(pads[3] as Pad).setText(EMPTY_TEXT);
				(pads[4] as Pad).setText(EMPTY_TEXT);
			}
			else
			{
				(pads[4] as Pad).setText(SERVE_TEXT);
				(pads[3] as Pad).setText(EMPTY_TEXT);
				(pads[2] as Pad).setText(EMPTY_TEXT);
				(pads[1] as Pad).setText(EMPTY_TEXT);
				(pads[0] as Pad).setText(EMPTY_TEXT);
			}
			
		}
		
		protected function startServe(player:Player):void
		{
			if (ball.motion)
			{
				ball.motion.kill();
			}
			
			var homePad:Pad;
			var reachPad:Pad;
			
			if (player.playerNum == 0)
			{
				homePad = pads[0];
				reachPad = pads[1];

			}
			else
			{
				homePad = pads[4];
				reachPad = pads[3];
			}
			
			//calculate initial time interval for the serve according to constants
			ball.speed = ball.MAX_SPEED;
			
			var tweenVars:TweenMaxVars = new TweenMaxVars( { x:reachPad.x , repeat: 1, yoyo:true, ease:Quad.easeInOut } );
			tweenVars.onComplete(serveCycle, [reachPad.x]);
			
			ball.motion = TweenMax.to(ball, serveIntervalSpeed, tweenVars);
			player.state = Player.STATE_SERVE;
			homePad.setText(EMPTY_TEXT);
			reachPad.setText(HIT_TEXT);
			
			soundManager.playServeHum(speedLevel);
		}
		
		protected function serveCycle (toX:int=0):void
		{
			
			
			if (speedLevel < MAX_SPEED_LEVEL)
			{
				
				speedLevel++;
				
				soundManager.playServeHum(speedLevel);
				
				for each (var pad:Pad in pads)
				{
					pad.shrink();
				}
			}
			
			ball.speed = ball.MAX_SPEED + ball.SPEED_UNIT * speedLevel;
			
			var tweenVars:TweenMaxVars = new TweenMaxVars( { x:toX , repeat: 1, yoyo:true, ease:Quad.easeInOut } );
			tweenVars.onComplete(serveCycle, [toX]);
			
			ball.motion = TweenMax.to(ball, serveIntervalSpeed, tweenVars);
			
			
			//trace ("Serve Cycle at speedlevel: " + speedLevel + " with ball.speed = " + ball.speed + " and intervalSpeed = " + serveIntervalSpeed);
		}
		
		protected function serve(player:Player):void
		{
		
			volley(player);
			
			soundManager.stopServeHum();
			soundManager.addHum(speedLevel);
			
			(pads[0] as Pad).setText(HIT_TEXT);
			(pads[1] as Pad).setText(START_CATCH_TEXT);
			(pads[2] as Pad).setText(SMASH_TEXT);
			(pads[3] as Pad).setText(START_CATCH_TEXT);
			(pads[4] as Pad).setText(HIT_TEXT);
			
			
		}
		
		protected function volley(player:Player):void
		{
			if (ball.motion)
				ball.motion.kill();
			
			var mvt:String;
			if (player.playerNum == 0)
				mvt = "+" + VOLLEY_DISTANCE;
			else
				mvt = "-" + VOLLEY_DISTANCE;
				
			var speed:Number = ball.speed / 1000;
			
			ball.motion = TweenMax.to(ball, speed, { x:mvt } );
			
			player.state = Player.STATE_ATTACK;
			player.opponent.state = Player.STATE_DEFENSE;
			
			soundManager.playTone();
		}
		
		protected function tryCatch(player:Player):void
		{
			player.state = Player.STATE_CATCHING;
			
			soundManager.playCatchTone();
			
			var homePad:Pad;
			var reachPad:Pad;
			
			if (player.playerNum == 0)
			{
				homePad = pads[0];
				reachPad = pads[1];

			}
			else
			{
				homePad = pads[4];
				reachPad = pads[3];
			}
			
			homePad.setText(END_CATCH_TEXT);
			reachPad.setText(EMPTY_TEXT);
		}
		
		protected function catchBall(player:Player):void
		{
			var homePad:Pad;
			
			if (player.playerNum == 0)
			{
				homePad = pads[0];
			}
			else
			{
				homePad = pads[4];
			}
			
			//tween ball to center of home pad
			ball.motion = TweenMax.to(ball, 0.3, { x: homePad.x } );
			
			//prep new serve
			prepServe(player);
		}
		
		protected function boostBall():void
		{
			soundManager.upgradeCurrentHum();
			soundManager.playCrash();
			
			speedLevel -= BOOST_FACTOR; //this allows for faster than normally possible speeds.
			
			ball.speed = ball.MAX_SPEED + ball.SPEED_UNIT * speedLevel;
			
			ball.boostBump();
			ball.motion.duration = ball.speed / 1000;
		}
		
		protected function get serveIntervalSpeed():Number
		{
			var value:Number = (ball.speed * (pads[1].x - pads[0].x) / VOLLEY_DISTANCE) / 1000;
			var factor:Number = 0;
			return value + factor;
		}
		
		protected function playerWins(player:Player):void
		{
			//disable keyboard
			this.stage.removeEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			this.stage.removeEventListener(KeyboardEvent.KEY_UP, handleKeyUp);
			
			//kill ball out timer
			ballOutTimer.stop();
			
			//stop sounds
			soundManager.stopHums();
			
			//show message
			var winMessage:PixelTextField = new PixelTextField();
			
			var winFormat:TextFormat = new TextFormat();
			winFormat.size = 200;
			winFormat.color = 0xffffff;
			winMessage.defaultTextFormat = winFormat;
			
			if (player.playerNum == 0)
				winMessage.text = "Left WINS";
			else
				winMessage.text = "Right WINS";
			
			

			winMessage.x = stage.stageWidth / 2 - winMessage.textWidth / 2;
			
			winMessage.y = 150;
			
			this.addChild(winMessage);
			
			//start reset timer
			var resetTimer:Timer = new Timer(5000,1);
			resetTimer.addEventListener(TimerEvent.TIMER_COMPLETE, gotoMenu);
			resetTimer.start();
		}
		
		protected function gotoMenu(te:TimerEvent):void
		{
			
			
			this.dispatchEvent(new Event("endGameplay"));
			this.parent.removeChild(this);
		}
	}

}