package ar.com.unobtainium.tonetennis.scenes 
{
	import ar.com.unobtainium.tonetennis.elements.Pad;
	import ar.com.unobtainium.tonetennis.elements.PixelTextField;
	import ar.com.unobtainium.tonetennis.sound.SoundMan;
	import com.greensock.data.TweenMaxVars;
	import com.greensock.events.TweenEvent;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.TimerEvent;
	import flash.text.TextFormat;
	import flash.utils.Timer;
	
	import com.greensock.*;
	import com.greensock.easing.*;
	
	
	/**
	 * Class to show the title and allow to start a 1 vs 1 game, allowing players to choose how many points must be played.
	 * 
	 * @author Balthazar Auger
	 */
	public class Menu extends Sprite 
	{
		
		public var pads:Array = [110, 380, 640, 900, 1170];
		public var padColors:Array = [0xffd580, 0xcccccc, 0xbefa8b, 0xcccccc, 0x64e0ff];
		public const BG_COLOR:uint = 0x333333;
		
		public var title:PixelTextField;
		
		public var soundManager:SoundMan;
		
		public var toPoints:int = 5;
		protected var pointSettings:Array = [5, 10, 15];
		protected var pointSettingsIndex:int = 0;
		
		protected var keys:Object = { };
		
		protected var p1Ready:Boolean = false;
		protected var p2Ready:Boolean = false;
		
		public var startTimer:Timer = new Timer(1000, 3);
		
		public var countdownText:PixelTextField;
		
		public function Menu() 
		{
			super();

			addEventListener(Event.ADDED_TO_STAGE, sceneAdded);

		}
		
		/**
		 * Menu Entry point
		 * @param	evt
		 */
		protected function sceneAdded(evt:Event):void
		{
			draw();
			
			initKeyboard();
			
			//startMenu();
		}
		
		protected function draw():void
		{
			//draw background
			var bg:Sprite = new Sprite();
			bg.graphics.beginFill(BG_COLOR);
			bg.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			bg.graphics.endFill();
			this.addChild(bg);
			
			//add pads
			for (var i:int = 0; i < pads.length;i++)
			{
				var pad:Pad = new Pad(padColors[i]);
				pad.x = pads[i];
				this.addChild(pad);
				pads[i] = pad;
			}
			
			(pads[0] as Pad).setText("READY?");
			(pads[2] as Pad).setText("5 PTS");
			(pads[4] as Pad).setText("READY?");
			
			//create sound manager
			soundManager = new SoundMan();
			
			// draw title
			title = new PixelTextField();
			var titleShadow:PixelTextField = new PixelTextField();
			
			var titleFormat:TextFormat = new TextFormat();
			titleFormat.size = 150;
			titleFormat.color = 0xffffff;
			
			var shadowFormat:TextFormat = new TextFormat();
			shadowFormat.size = 150;
			shadowFormat.color = 0x000000;
			
			title.defaultTextFormat = titleFormat;
			titleShadow.defaultTextFormat = shadowFormat;
			
			title.text = titleShadow.text = "ToneTennis";
			title.x = stage.stageWidth / 2 - title.textWidth / 2;
			title.y = 50;
			
			titleShadow.x = title.x + 10;
			titleShadow.y = title.y + 10;
			
			this.addChild(titleShadow);
			this.addChild(title);
			
			countdownText = new PixelTextField();
			var countdownFormat:TextFormat = new TextFormat();
			countdownFormat.size = 200;
			countdownFormat.color = 0x000000;
			countdownText.defaultTextFormat = countdownFormat;
			
			countdownText.y = title.y + title.height + 30;
			countdownText.text = " ";
			countdownText.x = stage.stageWidth / 2 - countdownText.textWidth / 2;
			
			this.addChild(countdownText);
		}
		
		/**
		 * KEYBOARD INPUT
		 */
		protected function initKeyboard():void
		{
			this.stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			this.stage.addEventListener(KeyboardEvent.KEY_UP, handleKeyUp);
		}
		
		protected function handleKeyDown(ke:KeyboardEvent):void
		{
			//trace (ke.keyCode + " down");
			 
			if( ! Boolean(ke.keyCode in keys)) {  
				keys[ke.keyCode] = true;
				

				if ((ke.keyCode >= 49) && (ke.keyCode <= 53))
				{
					actOnPadDown(ke.keyCode - 49);					
				}
			}	
		}
		
		protected function handleKeyUp(ke:KeyboardEvent):void
		{
			//trace (ke.keyCode + " up");
			delete keys[ke.keyCode];
			
			if ((ke.keyCode >= 49) && (ke.keyCode <= 53))
			{
				actOnPadUp(ke.keyCode - 49);
			}
			
		}
		
		protected function actOnPadDown(padNum:int):void
		{	
			//satisfy action feedback
			var pressedPad:Pad = (pads[padNum] as Pad);
			pressedPad.press();
			
			switch (padNum)
			{
				case 0: //left player home pad
					playerReady(1);
					break;
				case 2: // point settings pad
					changePointsSetting();
					break;
				case 4: // right player home pad
					playerReady(2);
					break;
			}
		}
		
		protected function actOnPadUp(padNum:int):void
		{	
			//satisfy action feedback
			var releasedPad:Pad = (pads[padNum] as Pad);
			releasedPad.release();
			
			switch (padNum)
			{
				case 0: //left player home pad
					playerUnready(1);
					break;
				case 4: // right player home pad
					playerUnready(2);
					break;
			}
		}
		
		public function playerReady(p:int):void
		{
			if (p == 1)
			{
				p1Ready = true;
				(pads[0] as Pad).setText("READY!");
			}
			else
			{
				p2Ready = true;
				(pads[4] as Pad).setText("READY!");
			}
			
			if (p1Ready && p2Ready)
			{
				startTimer.addEventListener(TimerEvent.TIMER_COMPLETE, leavePads);
				startTimer.addEventListener(TimerEvent.TIMER, countdown);
				startTimer.start();
				countdownText.text = "3";
			}
		}
		
		protected function countdown(te:TimerEvent):void
		{
			var number:int = 3 - startTimer.currentCount;
			countdownText.text = number.toString();
		}
		
		public function playerUnready(p:int):void
		{
			if (p == 1)
			{
				p1Ready = false;
				(pads[0] as Pad).setText("READY?");
			}
			else
			{
				p2Ready = false;
				(pads[4] as Pad).setText("READY?");
			}
			
			countdownText.text = " ";
			startTimer.reset();
		}
		
		public function changePointsSetting():void
		{
			pointSettingsIndex++;
			
			toPoints = pointSettings[pointSettingsIndex % pointSettings.length];
			
			(pads[2] as Pad).setText(toPoints.toString() + " PTS");
		}
		
		public function leavePads(te:TimerEvent):void
		{
			countdownText.text = "STEP OFF";
			
			startTimer.stop();
			startTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, leavePads);
			
			this.stage.removeEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown);
			this.stage.removeEventListener(KeyboardEvent.KEY_UP, handleKeyUp);
			this.stage.addEventListener(KeyboardEvent.KEY_UP, stepOff);
		}
		
		protected function stepOff(ke:KeyboardEvent):void
		{
			//trace (ke.keyCode + " up");
			delete keys[ke.keyCode];
			
			if ((!keys[49]) && (!keys[53]))
			{
				this.stage.removeEventListener(KeyboardEvent.KEY_UP, stepOff);
				this.dispatchEvent(new Event("startGameplay"));
			
				this.parent.removeChild(this);
			}
			
		}
	
		
	}

}